module;

#include <vector>

export module traveler;

import tile;

export class Traveler {
  public:
	int row;
	int col;
	Direction direction;

	Traveler(int row, int col, Direction direction) : row{row}, col{col}, direction{direction} {}

	void travel(std::vector<std::vector<Tile>> &grid);

  private:
	bool step(std::vector<std::vector<Tile>> &grid);
};

void
Traveler::travel(std::vector<std::vector<Tile>> &grid) {
	while (0 <= row && row < grid.size() && 0 <= col && col < grid[row].size()) {
		if (!step(grid)) {
			break;
		}
	}
}

bool
Traveler::step(std::vector<std::vector<Tile>> &grid) {
	Tile &current_tile = grid[row][col];

	Direction entry = opposite(direction);
	Direction const next = current_tile.connection(entry);

	if (entry == next) {
		return false;
	}

	current_tile.remove_connection(entry);
	direction = next;

	switch (direction) {
	case Direction::East: col += 1; break;
	case Direction::North: row -= 1; break;
	case Direction::West: col -= 1; break;
	case Direction::South: row += 1; break;
	default: std::unreachable();
	}

	return true;
}
